package java.awt;

import java.io.ObjectStreamException;
import java.lang.reflect.Field;
import java.util.Collections;
import java.util.StringTokenizer;
import java.awt.event.KeyEvent;
import java.util.HashMap;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import sun.awt.AppContext;
import java.util.Map;
import java.io.Serializable;

public class AWTKeyStroke implements Serializable
{
    static final long serialVersionUID = -6430539691155161871L;
    private static Map<String, Integer> modifierKeywords;
    private static VKCollection vks;
    private static Object APP_CONTEXT_CACHE_KEY;
    private static AWTKeyStroke APP_CONTEXT_KEYSTROKE_KEY;
    private char keyChar;
    private int keyCode;
    private int modifiers;
    private boolean onKeyRelease;
    
    private static Class<AWTKeyStroke> getAWTKeyStrokeClass() {
        Class<Object> clazz = (Class)AppContext.getAppContext().get(AWTKeyStroke.class);
        if (clazz == null) {
            clazz = AWTKeyStroke.class;
            AppContext.getAppContext().put(AWTKeyStroke.class, AWTKeyStroke.class);
            return clazz;
        }
        return clazz;
    }
    
    protected AWTKeyStroke() {
        this.keyChar = '\uffff';
        this.keyCode = 0;
    }
    
    protected AWTKeyStroke(char keyChar, int keyCode, int modifiers, boolean onKeyRelease) {
        this.keyChar = '\uffff';
        this.keyCode = 0;
        this.keyChar = keyChar;
        this.keyCode = keyCode;
        this.modifiers = modifiers;
        this.onKeyRelease = onKeyRelease;
    }
    
    protected static void registerSubclass(Class<?> clazz) {
        if (clazz == null) {
            throw new IllegalArgumentException("subclass cannot be null");
        }
        synchronized (AWTKeyStroke.class) {
            Class<Object> clazz2 = (Class)AppContext.getAppContext().get(AWTKeyStroke.class);
            if (clazz2 != null && clazz2.equals(clazz)) {
                return;
            }
        }
        if (!AWTKeyStroke.class.isAssignableFrom(clazz)) {
            throw new ClassCastException("subclass is not derived from AWTKeyStroke");
        }
        Constructor<Object> ctor = getCtor(clazz);
        String s = "subclass could not be instantiated";
        if (ctor == null) {
            throw new IllegalArgumentException(s);
        }
        try {
            if ((AWTKeyStroke)ctor.newInstance((Object[])null) == null) {
                throw new IllegalArgumentException(s);
            }
        }
        catch (NoSuchMethodError noSuchMethodError) {
            throw new IllegalArgumentException(s);
        }
        catch (ExceptionInInitializerError exceptionInInitializerError) {
            throw new IllegalArgumentException(s);
        }
        catch (InstantiationException ex) {
            throw new IllegalArgumentException(s);
        }
        catch (IllegalAccessException ex2) {
            throw new IllegalArgumentException(s);
        }
        catch (InvocationTargetException ex3) {
            throw new IllegalArgumentException(s);
        }
        synchronized (AWTKeyStroke.class) {
            AppContext.getAppContext().put(AWTKeyStroke.class, clazz);
            AppContext.getAppContext().remove(AWTKeyStroke.APP_CONTEXT_CACHE_KEY);
            AppContext.getAppContext().remove(AWTKeyStroke.APP_CONTEXT_KEYSTROKE_KEY);
        }
        return;
    }
    
    private static Constructor getCtor(final Class clazz) {
        return (Constructor)AccessController.doPrivileged(new PrivilegedAction<Constructor>() {
            @Override
            public Constructor run() {
                try {
                    Constructor<Object> declaredConstructor = clazz.getDeclaredConstructor((Class[])null);
                    if (declaredConstructor != null) {
                        declaredConstructor.setAccessible(true);
                        return declaredConstructor;
                    }
                    return declaredConstructor;
                }
                catch (SecurityException ex) {}
                catch (NoSuchMethodException ex2) {}
                return null;
            }
        });
    }
    
    private static synchronized AWTKeyStroke getCachedStroke(char keyChar, int keyCode, int n, boolean onKeyRelease) {
        Map<AWTKeyStroke, AWTKeyStroke> value = (Map)AppContext.getAppContext().get(AWTKeyStroke.APP_CONTEXT_CACHE_KEY);
        AWTKeyStroke value2 = (AWTKeyStroke)AppContext.getAppContext().get(AWTKeyStroke.APP_CONTEXT_KEYSTROKE_KEY);
        if (value == null) {
            value = new HashMap();
            AppContext.getAppContext().put(AWTKeyStroke.APP_CONTEXT_CACHE_KEY, value);
        }
        if (value2 == null) {
            try {
                value2 = (AWTKeyStroke)getCtor(getAWTKeyStrokeClass()).newInstance((Object[])null);
                AppContext.getAppContext().put(AWTKeyStroke.APP_CONTEXT_KEYSTROKE_KEY, value2);
            }
            catch (InstantiationException ex) {
                assert false;
            }
            catch (IllegalAccessException ex2) {
                assert false;
            }
            catch (InvocationTargetException ex3) {
                assert false;
            }
        }
        value2.keyChar = keyChar;
        value2.keyCode = keyCode;
        value2.modifiers = mapNewModifiers(mapOldModifiers(n));
        value2.onKeyRelease = onKeyRelease;
        AWTKeyStroke awtKeyStroke = (AWTKeyStroke)value.get(value2);
        if (awtKeyStroke == null) {
            awtKeyStroke = value2;
            value.put(awtKeyStroke, awtKeyStroke);
            AppContext.getAppContext().remove(AWTKeyStroke.APP_CONTEXT_KEYSTROKE_KEY);
            return awtKeyStroke;
        }
        return awtKeyStroke;
    }
    
    public static AWTKeyStroke getAWTKeyStroke(char c) {
        return getCachedStroke(c, 0, 0, false);
    }
    
    public static AWTKeyStroke getAWTKeyStroke(Character c, int n) {
        if (c == null) {
            throw new IllegalArgumentException("keyChar cannot be null");
        }
        return getCachedStroke((char)c, 0, n, false);
    }
    
    public static AWTKeyStroke getAWTKeyStroke(int n, int n2, boolean b) {
        return getCachedStroke('\uffff', n, n2, b);
    }
    
    public static AWTKeyStroke getAWTKeyStroke(int n, int n2) {
        return getCachedStroke('\uffff', n, n2, false);
    }
    
    public static AWTKeyStroke getAWTKeyStrokeForEvent(KeyEvent keyEvent) {
        int id = keyEvent.getID();
        switch (id) {
            case 401:
            case 402: {
                return getCachedStroke('\uffff', keyEvent.getKeyCode(), keyEvent.getModifiers(), id == 402);
            }
            case 400: {
                return getCachedStroke(keyEvent.getKeyChar(), 0, keyEvent.getModifiers(), false);
            }
            default: {
                return null;
            }
        }
    }
    
    public static AWTKeyStroke getAWTKeyStroke(String str) {
        if (str == null) {
            throw new IllegalArgumentException("String cannot be null");
        }
        StringTokenizer stringTokenizer = new StringTokenizer(str, " ");
        int n = 0;
        boolean n2 = false;
        int n3 = 0;
        int n4 = 0;
        synchronized (AWTKeyStroke.class) {
            if (AWTKeyStroke.modifierKeywords == null) {
                HashMap m = new HashMap(8, 1.0f);
                m.put("shift", Integer.valueOf(65));
                m.put("control", Integer.valueOf(130));
                m.put("ctrl", Integer.valueOf(130));
                m.put("meta", Integer.valueOf(260));
                m.put("alt", Integer.valueOf(520));
                m.put("altGraph", Integer.valueOf(8224));
                m.put("button1", Integer.valueOf(1024));
                m.put("button2", Integer.valueOf(2048));
                m.put("button3", Integer.valueOf(4096));
                AWTKeyStroke.modifierKeywords = Collections.synchronizedMap(m);
            }
        }
        int countTokens = stringTokenizer.countTokens();
        int i = 1;
        while (i <= countTokens) {
            String nextToken = stringTokenizer.nextToken();
            if (n3 != 0) {
                if (nextToken.length() != 1 || i != countTokens) {
                    throw new IllegalArgumentException("String formatted incorrectly");
                }
                return getCachedStroke(nextToken.charAt(0), 0, n, false);
            }
            else if (n4 != 0 || n2 || i == countTokens) {
                if (i != countTokens) {
                    throw new IllegalArgumentException("String formatted incorrectly");
                }
                return getCachedStroke('\uffff', getVKValue("VK_" + nextToken), n, (boolean)n2);
            }
            else {
                if (nextToken.equals("released")) {
                    n2 = true;
                }
                else if (nextToken.equals("pressed")) {
                    n4 = 1;
                }
                else if (nextToken.equals("typed")) {
                    n3 = 1;
                }
                else {
                    Integer n5 = (Integer)AWTKeyStroke.modifierKeywords.get(nextToken);
                    if (n5 != null) {
                        n |= (int)n5;
                    }
                    else {
                        throw new IllegalArgumentException("String formatted incorrectly");
                    }
                }
                ++i;
            }
        }
        throw new IllegalArgumentException("String formatted incorrectly");
    }
    
    private static VKCollection getVKCollection() {
        if (AWTKeyStroke.vks == null) {
            AWTKeyStroke.vks = new VKCollection();
        }
        return AWTKeyStroke.vks;
    }
    
    private static int getVKValue(String name) {
        VKCollection vkCollection = getVKCollection();
        Integer n = vkCollection.findCode(name);
        if (n == null) {
            int int1;
            try {
                int1 = KeyEvent.class.getField(name).getInt(KeyEvent.class);
            }
            catch (NoSuchFieldException ex) {
                throw new IllegalArgumentException("String formatted incorrectly");
            }
            catch (IllegalAccessException ex2) {
                throw new IllegalArgumentException("String formatted incorrectly");
            }
            n = Integer.valueOf(int1);
            vkCollection.put(name, n);
        }
        return (int)n;
    }
    
    public final char getKeyChar() {
        return this.keyChar;
    }
    
    public final int getKeyCode() {
        return this.keyCode;
    }
    
    public final int getModifiers() {
        return this.modifiers;
    }
    
    public final boolean isOnKeyRelease() {
        return this.onKeyRelease;
    }
    
    public final int getKeyEventType() {
        if (this.keyCode == 0) {
            return 400;
        }
        return this.onKeyRelease ? 402 : 401;
    }
    
    @Override
    public int hashCode() {
        return (this.keyChar + 1) * (2 * (this.keyCode + 1)) * (this.modifiers + 1) + (this.onKeyRelease ? 1 : 2);
    }
    
    @Override
    public final boolean equals(Object o) {
        if ((o instanceof AWTKeyStroke)) {
            AWTKeyStroke awtKeyStroke = (AWTKeyStroke)o;
            return awtKeyStroke.keyChar == this.keyChar && awtKeyStroke.keyCode == this.keyCode && awtKeyStroke.onKeyRelease == this.onKeyRelease && awtKeyStroke.modifiers == this.modifiers;
        }
        return false;
    }
    
    @Override
    public String toString() {
        if (this.keyCode == 0) {
            return getModifiersText(this.modifiers) + "typed " + this.keyChar;
        }
        return getModifiersText(this.modifiers) + (this.onKeyRelease ? "released" : "pressed") + " " + getVKText(this.keyCode);
    }
    
    static String getModifiersText(int n) {
        StringBuilder sb = new StringBuilder();
        if ((n & 0x40) != 0x0) {
            sb.append("shift ");
        }
        if ((n & 0x80) != 0x0) {
            sb.append("ctrl ");
        }
        if ((n & 0x100) != 0x0) {
            sb.append("meta ");
        }
        if ((n & 0x200) != 0x0) {
            sb.append("alt ");
        }
        if ((n & 0x2000) != 0x0) {
            sb.append("altGraph ");
        }
        if ((n & 0x400) != 0x0) {
            sb.append("button1 ");
        }
        if ((n & 0x800) != 0x0) {
            sb.append("button2 ");
        }
        if ((n & 0x1000) != 0x0) {
            sb.append("button3 ");
        }
        return sb.toString();
    }
    
    static String getVKText(int i) {
        VKCollection vkCollection = getVKCollection();
        Integer value = Integer.valueOf(i);
        String name = vkCollection.findName(value);
        if (name != null) {
            return name.substring(3);
        }
        int n = 25;
        Field[] declaredFields = KeyEvent.class.getDeclaredFields();
        for (int j = 0; j < declaredFields.length; ++j) {
            try {
                if (declaredFields[j].getModifiers() == n && declaredFields[j].getType() == Integer.TYPE && declaredFields[j].getName().startsWith("VK_") && declaredFields[j].getInt(KeyEvent.class) == i) {
                    String name2 = declaredFields[j].getName();
                    vkCollection.put(name2, value);
                    return name2.substring(3);
                }
            }
            catch (IllegalAccessException ex) {
                assert false;
            }
        }
        return "UNKNOWN";
    }
    
    protected Object readResolve() throws ObjectStreamException {
        synchronized (AWTKeyStroke.class) {
            if (this.getClass().equals(getAWTKeyStrokeClass())) {
                return getCachedStroke(this.keyChar, this.keyCode, this.modifiers, this.onKeyRelease);
            }
        }
        return this;
    }
    
    private static int mapOldModifiers(int n) {
        if ((n & 0x1) != 0x0) {
            n |= 0x40;
        }
        if ((n & 0x8) != 0x0) {
            n |= 0x200;
        }
        if ((n & 0x20) != 0x0) {
            n |= 0x2000;
        }
        if ((n & 0x2) != 0x0) {
            n |= 0x80;
        }
        if ((n & 0x4) != 0x0) {
            n |= 0x100;
        }
        n &= 0x3FC0;
        return n;
    }
    
    private static int mapNewModifiers(int n) {
        if ((n & 0x40) != 0x0) {
            n |= 0x1;
        }
        if ((n & 0x200) != 0x0) {
            n |= 0x8;
        }
        if ((n & 0x2000) != 0x0) {
            n |= 0x20;
        }
        if ((n & 0x80) != 0x0) {
            n |= 0x2;
        }
        if ((n & 0x100) != 0x0) {
            n |= 0x4;
        }
        return n;
    }
    
    static {
        AWTKeyStroke.APP_CONTEXT_CACHE_KEY = new Object();
        AWTKeyStroke.APP_CONTEXT_KEYSTROKE_KEY = new AWTKeyStroke();
        Toolkit.loadLibraries();
    }
}
